setGeneric("getNobs",function(x)standardGeneric("getNobs"))
setGeneric("seekData",function(x)standardGeneric("seekData"))
setGeneric("readData",function(x,n)standardGeneric("readData"))
setGeneric("readSlice",function(x,rows,cols)standardGeneric("readSlice"))
setGeneric("readChunk",function(x,nrows,cols)standardGeneric("readChunk"))
setGeneric("codebookEntry",function(x,weights=NULL,unweighted=TRUE,...) standardGeneric("codebookEntry"))
# setGeneric("codebookStatsCateg",function(x,by=NULL) standardGeneric("codebookStatsCateg"))
# setGeneric("codebookStatsMetric",function(x,by=NULL) standardGeneric("codebookStatsMetric"))
setGeneric("relabel4",function(x,...) standardGeneric("relabel4"))
